@@ -1,4 +1,5 @@
 use libc::size_t;
+use std::convert::TryFrom;
 use std::io::Cursor;
 use std::ptr::null;
 use std::slice;
@@ -13,7 +14,7 @@ use rustls::{
 use rustls_pemfile::{certs, pkcs8_private_keys, rsa_private_keys};
 
 use crate::error::rustls_result;
-use crate::rslice::rustls_slice_bytes;
+use crate::rslice::{rustls_slice_bytes, rustls_str};
 use crate::{
     ffi_panic_boundary, try_mut_from_ptr, try_ref_from_ptr, try_slice, ArcCastPtr, BoxCastPtr,
     CastConstPtr, CastPtr,
@@ -46,11 +47,14 @@ impl rustls_certificate {
     ) -> rustls_result {
         ffi_panic_boundary! {
             let cert = try_ref_from_ptr!(cert);
-            let out_der_data: &mut *const u8 = try_mut_from_ptr!(out_der_data);
-            let out_der_len: &mut size_t = try_mut_from_ptr!(out_der_len);
+            if out_der_data.is_null() || out_der_len.is_null() {
+                return NullParameter
+            }
             let der = cert.as_ref();
-            *out_der_data = der.as_ptr();
-            *out_der_len = der.len();
+            unsafe {
+                *out_der_data = der.as_ptr();
+                *out_der_len = der.len();
+            }
             rustls_result::Ok
         }
     }
@@ -83,6 +87,22 @@ impl rustls_supported_ciphersuite {
     }
 }
 
+/// Returns the name of the ciphersuite as a `rustls_str`. If the provided
+/// ciphersuite is invalid, the rustls_str will contain the empty string. The
+/// lifetime of the `rustls_str` is the lifetime of the program, it does not
+/// need to be freed.
+#[no_mangle]
+pub extern "C" fn rustls_supported_ciphersuite_get_name(
+    supported_ciphersuite: *const rustls_supported_ciphersuite,
+) -> rustls_str<'static> {
+    let supported_ciphersuite = try_ref_from_ptr!(supported_ciphersuite);
+    let s = supported_ciphersuite.suite().as_str().unwrap_or("");
+    match rustls_str::try_from(s) {
+        Ok(s) => s,
+        Err(_) => rustls_str::from_str_unchecked(""),
+    }
+}
+
 /// Return the length of rustls' list of supported cipher suites.
 #[no_mangle]
 pub extern "C" fn rustls_all_ciphersuites_len() -> usize {
@@ -123,6 +143,113 @@ pub extern "C" fn rustls_default_ciphersuites_get_entry(
     }
 }
 
+/// Rustls' list of supported cipher suites. This is an array of pointers, and
+/// its length is given by `RUSTLS_ALL_CIPHER_SUITES_LEN`. The pointers will
+/// always be valid. The contents and order of this array may change between
+/// releases.
+#[no_mangle]
+pub static mut RUSTLS_ALL_CIPHER_SUITES: [*const rustls_supported_ciphersuite; 9] = [
+    &rustls::cipher_suite::TLS13_AES_256_GCM_SHA384 as *const SupportedCipherSuite as *const _,
+    &rustls::cipher_suite::TLS13_AES_128_GCM_SHA256 as *const SupportedCipherSuite as *const _,
+    &rustls::cipher_suite::TLS13_CHACHA20_POLY1305_SHA256 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256
+        as *const SupportedCipherSuite as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256
+        as *const SupportedCipherSuite as *const _,
+];
+
+/// The length of the array `RUSTLS_ALL_CIPHER_SUITES`.
+#[no_mangle]
+pub static RUSTLS_ALL_CIPHER_SUITES_LEN: usize = unsafe { RUSTLS_ALL_CIPHER_SUITES.len() };
+
+/// Rustls' list of default cipher suites. This is an array of pointers, and
+/// its length is given by `RUSTLS_DEFAULT_CIPHER_SUITES_LEN`. The pointers
+/// will always be valid. The contents and order of this array may change
+/// between releases.
+#[no_mangle]
+pub static mut RUSTLS_DEFAULT_CIPHER_SUITES: [*const rustls_supported_ciphersuite; 9] = [
+    &rustls::cipher_suite::TLS13_AES_256_GCM_SHA384 as *const SupportedCipherSuite as *const _,
+    &rustls::cipher_suite::TLS13_AES_128_GCM_SHA256 as *const SupportedCipherSuite as *const _,
+    &rustls::cipher_suite::TLS13_CHACHA20_POLY1305_SHA256 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256
+        as *const SupportedCipherSuite as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256 as *const SupportedCipherSuite
+        as *const _,
+    &rustls::cipher_suite::TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256
+        as *const SupportedCipherSuite as *const _,
+];
+
+/// The length of the array `RUSTLS_DEFAULT_CIPHER_SUITES`.
+#[no_mangle]
+pub static RUSTLS_DEFAULT_CIPHER_SUITES_LEN: usize = unsafe { RUSTLS_DEFAULT_CIPHER_SUITES.len() };
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use std::slice;
+    use std::str;
+
+    #[test]
+    fn all_cipher_suites_arrays() {
+        assert_eq!(RUSTLS_ALL_CIPHER_SUITES_LEN, ALL_CIPHER_SUITES.len());
+        for (original, ffi) in ALL_CIPHER_SUITES
+            .iter()
+            .zip(unsafe { RUSTLS_ALL_CIPHER_SUITES }.iter().copied())
+        {
+            let ffi_cipher_suite = try_ref_from_ptr!(ffi);
+            assert_eq!(original, ffi_cipher_suite);
+        }
+    }
+
+    #[test]
+    fn default_cipher_suites_arrays() {
+        assert_eq!(
+            RUSTLS_DEFAULT_CIPHER_SUITES_LEN,
+            DEFAULT_CIPHER_SUITES.len()
+        );
+        for (original, ffi) in DEFAULT_CIPHER_SUITES
+            .iter()
+            .zip(unsafe { RUSTLS_DEFAULT_CIPHER_SUITES }.iter().copied())
+        {
+            let ffi_cipher_suite = try_ref_from_ptr!(ffi);
+            assert_eq!(original, ffi_cipher_suite);
+        }
+    }
+
+    #[test]
+    fn ciphersuite_get_name() {
+        let suite = rustls_all_ciphersuites_get_entry(0);
+        let s = rustls_supported_ciphersuite_get_name(suite);
+        let want = "TLS13_AES_256_GCM_SHA384";
+        unsafe {
+            let got = str::from_utf8(slice::from_raw_parts(s.data as *const u8, s.len)).unwrap();
+            assert_eq!(want, got)
+        }
+    }
+
+    #[test]
+    fn test_all_ciphersuites_len() {
+        let len = rustls_all_ciphersuites_len();
+        assert!(len > 2);
+    }
+}
+
 /// The complete chain of certificates to send during a TLS handshake,
 /// plus a private key that matches the end-entity (leaf) certificate.
 /// Corresponds to `CertifiedKey` in the Rust API.
