@@ -185,7 +185,7 @@ typedef struct rustls_server_config_builder rustls_server_config_builder;
 /**
  * A read-only view of a slice of Rust byte slices.
  *
- * This is used to pass data from crustls to callback functions provided
+ * This is used to pass data from rustls-ffi to callback functions provided
  * by the user of the API. Because Vec and slice are not `#[repr(C)]`, we
  * provide access via a pointer to an opaque struct and an accessor method
  * that acts on that struct to get entries of type `rustls_slice_bytes`.
@@ -204,7 +204,7 @@ typedef struct rustls_slice_slice_bytes rustls_slice_slice_bytes;
  * strings). Like `rustls_str`, this guarantees that each string contains
  * UTF-8 and no NUL bytes. Strings are not NUL-terminated.
  *
- * This is used to pass data from crustls to callback functions provided
+ * This is used to pass data from rustls-ffi to callback functions provided
  * by the user of the API. Because Vec and slice are not `#[repr(C)]`, we
  * can't provide a straightforward `data` and `len` structure. Instead, we
  * provide access via a pointer to an opaque struct and accessor methods.
@@ -244,7 +244,7 @@ typedef struct rustls_str {
 /**
  * A read-only view on a Rust byte slice.
  *
- * This is used to pass data from crustls to callback functions provided
+ * This is used to pass data from rustls-ffi to callback functions provided
  * by the user of the API.
  * `len` indicates the number of bytes than can be safely read.
  *
@@ -276,7 +276,7 @@ typedef struct rustls_verify_server_cert_params {
   struct rustls_slice_bytes ocsp_response;
 } rustls_verify_server_cert_params;
 
-typedef rustls_result (*rustls_verify_server_cert_callback)(rustls_verify_server_cert_user_data userdata, const struct rustls_verify_server_cert_params *params);
+typedef uint32_t (*rustls_verify_server_cert_callback)(rustls_verify_server_cert_user_data userdata, const struct rustls_verify_server_cert_params *params);
 
 typedef size_t rustls_log_level;
 
@@ -351,7 +351,7 @@ typedef void *rustls_client_hello_userdata;
 /**
  * A read-only view on a Rust slice of 16-bit integers in platform endianness.
  *
- * This is used to pass data from crustls to callback functions provided
+ * This is used to pass data from rustls-ffi to callback functions provided
  * by the user of the API.
  * `len` indicates the number of bytes than can be safely read.
  *
@@ -380,7 +380,7 @@ typedef struct rustls_slice_u16 {
  * the call and may not be modified. Users of this API must copy any values that
  * they want to access when the callback returned.
  *
- * EXPERIMENTAL: this feature of crustls is likely to change in the future, as
+ * EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
  * the rustls library is re-evaluating their current approach to client hello handling.
  */
 typedef struct rustls_client_hello {
@@ -400,10 +400,11 @@ typedef struct rustls_client_hello {
  * NOTE:
  * - the passed in `hello` and all its values are only available during the
  *   callback invocations.
- * - the passed callback function must be implemented thread-safe, unless
- *   there is only a single config and session where it is installed.
+ * - the passed callback function must be safe to call multiple times concurrently
+ *   with the same userdata, unless there is only a single config and connection
+ *   where it is installed.
  *
- * EXPERIMENTAL: this feature of crustls is likely to change in the future, as
+ * EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
  * the rustls library is re-evaluating their current approach to client hello handling.
  */
 typedef const struct rustls_certified_key *(*rustls_client_hello_callback)(rustls_client_hello_userdata userdata, const struct rustls_client_hello *hello);
@@ -428,8 +429,9 @@ typedef void *rustls_session_store_userdata;
  * do a partial copy but instead remove the value from its store and
  * act as if it was never found.
  *
- * The callback should return != 0 to indicate that a value was retrieved
- * and written in its entirety into `buf`.
+ * The callback should return RUSTLS_RESULT_OK to indicate that a value was
+ * retrieved and written in its entirety into `buf`, or RUSTLS_RESULT_NOT_FOUND
+ * if no session was retrieved.
  *
  * When `remove_after` is != 0, the returned data needs to be removed
  * from the store.
@@ -439,7 +441,7 @@ typedef void *rustls_session_store_userdata;
  * NOTE: callbacks used in several sessions via a common config
  * must be implemented thread-safe.
  */
-typedef rustls_result (*rustls_session_store_get_callback)(rustls_session_store_userdata userdata, const struct rustls_slice_bytes *key, int remove_after, uint8_t *buf, size_t count, size_t *out_n);
+typedef uint32_t (*rustls_session_store_get_callback)(rustls_session_store_userdata userdata, const struct rustls_slice_bytes *key, int remove_after, uint8_t *buf, size_t count, size_t *out_n);
 
 /**
  * Prototype of a callback that can be installed by the application at the
@@ -448,15 +450,31 @@ typedef rustls_result (*rustls_session_store_get_callback)(rustls_session_store_
  * for later use is handed to the client/has been received from the server.
  * `userdata` will be supplied based on rustls_{client,server}_session_set_userdata.
  *
- * The callback should return != 0 to indicate that the value has been
- * successfully persisted in its store.
+ * The callback should return RUSTLS_RESULT_OK to indicate that a value was
+ * successfully stored, or RUSTLS_RESULT_IO on failure.
  *
  * NOTE: the passed in `key` and `val` are only available during the
  * callback invocation.
  * NOTE: callbacks used in several sessions via a common config
  * must be implemented thread-safe.
  */
-typedef rustls_result (*rustls_session_store_put_callback)(rustls_session_store_userdata userdata, const struct rustls_slice_bytes *key, const struct rustls_slice_bytes *val);
+typedef uint32_t (*rustls_session_store_put_callback)(rustls_session_store_userdata userdata, const struct rustls_slice_bytes *key, const struct rustls_slice_bytes *val);
+
+extern const struct rustls_supported_ciphersuite *RUSTLS_ALL_CIPHER_SUITES[9];
+
+extern const size_t RUSTLS_ALL_CIPHER_SUITES_LEN;
+
+extern const struct rustls_supported_ciphersuite *RUSTLS_DEFAULT_CIPHER_SUITES[9];
+
+extern const size_t RUSTLS_DEFAULT_CIPHER_SUITES_LEN;
+
+extern const uint16_t RUSTLS_ALL_VERSIONS[2];
+
+extern const size_t RUSTLS_ALL_VERSIONS_LEN;
+
+extern const uint16_t RUSTLS_DEFAULT_VERSIONS[2];
+
+extern const size_t RUSTLS_DEFAULT_VERSIONS_LEN;
 
 /**
  * Returns a static string containing the rustls-ffi version as well as the
@@ -480,6 +498,14 @@ rustls_result rustls_certificate_get_der(const struct rustls_certificate *cert,
  */
 uint16_t rustls_supported_ciphersuite_get_suite(const struct rustls_supported_ciphersuite *supported_ciphersuite);
 
+/**
+ * Returns the name of the ciphersuite as a `rustls_str`. If the provided
+ * ciphersuite is invalid, the rustls_str will contain the empty string. The
+ * lifetime of the `rustls_str` is the lifetime of the program, it does not
+ * need to be freed.
+ */
+struct rustls_str rustls_supported_ciphersuite_get_name(const struct rustls_supported_ciphersuite *supported_ciphersuite);
+
 /**
  * Return the length of rustls' list of supported cipher suites.
  */
@@ -646,12 +672,14 @@ struct rustls_client_config_builder *rustls_client_config_builder_new(void);
  * resulting rustls_client_config. Specify cipher suites in preference
  * order; the `cipher_suites` parameter must point to an array containing
  * `len` pointers to `rustls_supported_ciphersuite` previously obtained
- * from `rustls_all_ciphersuites_get_entry()`. Set the TLS protocol
- * versions to use when negotiating a TLS session.
+ * from `rustls_all_ciphersuites_get_entry()`, or to a provided array,
+ * RUSTLS_DEFAULT_CIPHER_SUITES or RUSTLS_ALL_CIPHER_SUITES. Set the TLS
+ * protocol versions to use when negotiating a TLS session.
  *
  * `tls_version` is the version of the protocol, as defined in rfc8446,
  * ch. 4.2.1 and end of ch. 5.1. Some values are defined in
- * `rustls_tls_version` for convenience.
+ * `rustls_tls_version` for convenience, and the arrays
+ * RUSTLS_DEFAULT_VERSIONS or RUSTLS_ALL_VERSIONS can be used directly.
  *
  * `versions` will only be used during the call and the application retains
  * ownership. `len` is the number of consecutive `uint16_t` pointed to by `versions`.
@@ -683,7 +711,7 @@ rustls_result rustls_client_config_builder_new_custom(const struct rustls_suppor
  *
  * If you intend to write a verifier that accepts all certificates, be aware
  * that special measures are required for IP addresses. Rustls currently
- * (0.20.0) doesn't support building a ClientSession with an IP address
+ * (0.20.0) doesn't support building a ClientConnection with an IP address
  * (because it's not a valid DnsNameRef). One workaround is to detect IP
  * addresses and rewrite them to `example.invalid`, and _also_ to disable
  * SNI via rustls_client_config_builder_set_enable_sni (IP addresses don't
@@ -742,7 +770,7 @@ void rustls_client_config_builder_set_enable_sni(struct rustls_client_config_bui
                                                  bool enable);
 
 /**
- * Provide the configuration a list of certificates where the session
+ * Provide the configuration a list of certificates where the connection
  * will select the first one that is compatible with the server's signature
  * verification capabilities. Clients that want to support both ECDSA and
  * RSA certificates will want the ECSDA to go first in the list.
@@ -787,7 +815,7 @@ void rustls_client_config_free(const struct rustls_client_config *config);
 /**
  * Create a new rustls_connection containing a client connection and return
  * it in the output parameter `out`. If this returns an error code, the
- * memory pointed to by `session_out` remains unchanged. If this returns a
+ * memory pointed to by `conn_out` remains unchanged. If this returns a
  * non-error, the memory pointed to by `conn_out` is modified to point at a
  * valid rustls_connection. The caller now owns the rustls_connection and must
  * call `rustls_connection_free` when done with it.
@@ -979,6 +1007,28 @@ rustls_result rustls_connection_read(struct rustls_connection *conn,
                                      size_t count,
                                      size_t *out_n);
 
+#if defined(DEFINE_READ_BUF)
+/**
+ * Read up to `count` plaintext bytes from the `rustls_connection` into `buf`.
+ * On success, store the number of bytes read in *out_n (this may be less
+ * than `count`). A success with *out_n set to 0 means "all bytes currently
+ * available have been read, but more bytes may become available after
+ * subsequent calls to rustls_connection_read_tls and
+ * rustls_connection_process_new_packets."
+ *
+ * This experimental API is only available when using a nightly Rust compiler
+ * and enabling the `read_buf` Cargo feature. It will be deprecated and later
+ * removed in future versions.
+ *
+ * Unlike with `rustls_connection_read`, this function may be called with `buf`
+ * pointing to an uninitialized memory buffer.
+ */
+rustls_result rustls_connection_read_2(struct rustls_connection *conn,
+                                       uint8_t *buf,
+                                       size_t count,
+                                       size_t *out_n);
+#endif
+
 /**
  * Free a rustls_connection. Calling with NULL is fine.
  * Must not be called twice with the same value.
@@ -993,7 +1043,7 @@ void rustls_connection_free(struct rustls_connection *conn);
  */
 void rustls_error(unsigned int result, char *buf, size_t len, size_t *out_n);
 
-bool rustls_result_is_cert_error(rustls_result result);
+bool rustls_result_is_cert_error(unsigned int result);
 
 /**
  * Return a rustls_str containing the stringified version of a log level.
@@ -1112,7 +1162,7 @@ rustls_result rustls_server_config_builder_set_alpn_protocols(struct rustls_serv
                                                               size_t len);
 
 /**
- * Provide the configuration a list of certificates where the session
+ * Provide the configuration a list of certificates where the connection
  * will select the first one that is compatible with the client's signature
  * verification capabilities. Servers that want to support both ECDSA and
  * RSA certificates will want the ECSDA to go first in the list.
@@ -1148,8 +1198,8 @@ void rustls_server_config_free(const struct rustls_server_config *config);
 /**
  * Create a new rustls_connection containing a server connection, and return it
  * in the output parameter `out`. If this returns an error code, the memory
- * pointed to by `session_out` remains unchanged. If this returns a non-error,
- * the memory pointed to by `session_out` is modified to point
+ * pointed to by `conn_out` remains unchanged. If this returns a non-error,
+ * the memory pointed to by `conn_out` is modified to point
  * at a valid rustls_connection. The caller now owns the rustls_connection
  * and must call `rustls_connection_free` when done with it.
  */
@@ -1161,7 +1211,7 @@ rustls_result rustls_server_connection_new(const struct rustls_server_config *co
  * and the length of that hostname in `out_n`. The string is stored in UTF-8
  * with no terminating NUL byte.
  * Returns RUSTLS_RESULT_INSUFFICIENT_SIZE if the SNI hostname is longer than `count`.
- * Returns Ok with *out_n == 0 if there is no SNI hostname available on this session
+ * Returns Ok with *out_n == 0 if there is no SNI hostname available on this connection
  * because it hasn't been processed yet, or because the client did not send SNI.
  * <https://docs.rs/rustls/0.20.0/rustls/server/struct.ServerConnection.html#method.sni_hostname>
  */
@@ -1171,8 +1221,8 @@ rustls_result rustls_server_connection_get_sni_hostname(const struct rustls_conn
                                                         size_t *out_n);
 
 /**
- * Register a callback to be invoked when a session created from this config
- * is seeing a TLS ClientHello message. If `userdata` has been set with
+ * Register a callback to be invoked when a connection created from this config
+ * sees a TLS ClientHello message. If `userdata` has been set with
  * rustls_connection_set_userdata, it will be passed to the callback.
  * Otherwise the userdata param passed to the callback will be NULL.
  *
@@ -1181,7 +1231,7 @@ rustls_result rustls_server_connection_get_sni_hostname(const struct rustls_conn
  * will overwrite the first registration. It is not permitted to pass a NULL
  * value for `callback`.
  *
- * EXPERIMENTAL: this feature of crustls is likely to change in the future, as
+ * EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
  * the rustls library is re-evaluating their current approach to client hello handling.
  * Installing a client_hello callback will replace any configured certified keys
  * and vice versa. Same holds true for the set_certified_keys variant.
@@ -1213,7 +1263,7 @@ rustls_result rustls_client_hello_select_certified_key(const struct rustls_clien
 /**
  * Register callbacks for persistence of TLS session IDs and secrets. Both
  * keys and values are highly sensitive data, containing enough information
- * to break the security of the sessions involved.
+ * to break the security of the connections involved.
  *
  * If `userdata` has been set with rustls_connection_set_userdata, it
  * will be passed to the callbacks. Otherwise the userdata param passed to
