@@ -28,8 +28,8 @@ use crate::session::{
     SessionStoreGetCallback, SessionStorePutCallback,
 };
 use crate::{
-    ffi_panic_boundary, try_arc_from_ptr, try_box_from_ptr, try_mut_from_ptr, try_mut_slice,
-    try_ref_from_ptr, try_slice, userdata_get, ArcCastPtr, BoxCastPtr, CastConstPtr, CastPtr,
+    ffi_panic_boundary, try_arc_from_ptr, try_box_from_ptr, try_mut_from_ptr, try_ref_from_ptr,
+    try_slice, userdata_get, ArcCastPtr, BoxCastPtr, CastConstPtr, CastPtr,
 };
 
 /// A server config being constructed. A builder can be modified by,
@@ -252,7 +252,7 @@ impl rustls_server_config_builder {
         }
     }
 
-    /// Provide the configuration a list of certificates where the session
+    /// Provide the configuration a list of certificates where the connection
     /// will select the first one that is compatible with the client's signature
     /// verification capabilities. Servers that want to support both ECDSA and
     /// RSA certificates will want the ECSDA to go first in the list.
@@ -329,8 +329,8 @@ impl rustls_server_config {
 
     /// Create a new rustls_connection containing a server connection, and return it
     /// in the output parameter `out`. If this returns an error code, the memory
-    /// pointed to by `session_out` remains unchanged. If this returns a non-error,
-    /// the memory pointed to by `session_out` is modified to point
+    /// pointed to by `conn_out` remains unchanged. If this returns a non-error,
+    /// the memory pointed to by `conn_out` is modified to point
     /// at a valid rustls_connection. The caller now owns the rustls_connection
     /// and must call `rustls_connection_free` when done with it.
     #[no_mangle]
@@ -359,7 +359,7 @@ impl rustls_server_config {
 /// and the length of that hostname in `out_n`. The string is stored in UTF-8
 /// with no terminating NUL byte.
 /// Returns RUSTLS_RESULT_INSUFFICIENT_SIZE if the SNI hostname is longer than `count`.
-/// Returns Ok with *out_n == 0 if there is no SNI hostname available on this session
+/// Returns Ok with *out_n == 0 if there is no SNI hostname available on this connection
 /// because it hasn't been processed yet, or because the client did not send SNI.
 /// <https://docs.rs/rustls/0.20.0/rustls/server/struct.ServerConnection.html#method.sni_hostname>
 #[no_mangle]
@@ -371,8 +371,12 @@ pub extern "C" fn rustls_server_connection_get_sni_hostname(
 ) -> rustls_result {
     ffi_panic_boundary! {
         let conn: &Connection = try_ref_from_ptr!(conn);
-        let write_buf: &mut [u8] = try_mut_slice!(buf, count);
-        let out_n: &mut size_t = try_mut_from_ptr!(out_n);
+        if buf.is_null() {
+            return NullParameter
+        }
+        if out_n.is_null() {
+            return NullParameter
+        }
         let server_connection = match conn.as_server() {
             Some(s) => s,
             _ => return rustls_result::InvalidParameter,
@@ -384,16 +388,21 @@ pub extern "C" fn rustls_server_connection_get_sni_hostname(
             },
         };
         let len: usize = sni_hostname.len();
-        if len > write_buf.len() {
+        if len > count {
+            unsafe {
+                *out_n = 0
+            }
             return rustls_result::InsufficientSize;
         }
-        write_buf[..len].copy_from_slice(sni_hostname.as_bytes());
-        *out_n = len;
+        unsafe {
+            std::ptr::copy_nonoverlapping(sni_hostname.as_ptr(), buf, len);
+            *out_n = len;
+        }
         rustls_result::Ok
     }
 }
 
-/// Choose the server certificate to be used for a session based on certificate
+/// Choose the server certificate to be used for a connection based on certificate
 /// type. Will pick the first CertfiedKey available that is suitable for
 /// the SignatureSchemes supported by the client.
 struct ResolvesServerCertFromChoices {
@@ -436,7 +445,7 @@ impl ResolvesServerCert for ResolvesServerCertFromChoices {
 /// the call and may not be modified. Users of this API must copy any values that
 /// they want to access when the callback returned.
 ///
-/// EXPERIMENTAL: this feature of crustls is likely to change in the future, as
+/// EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
 /// the rustls library is re-evaluating their current approach to client hello handling.
 #[repr(C)]
 pub struct rustls_client_hello<'a> {
@@ -462,10 +471,11 @@ pub type rustls_client_hello_userdata = *mut c_void;
 /// NOTE:
 /// - the passed in `hello` and all its values are only available during the
 ///   callback invocations.
-/// - the passed callback function must be implemented thread-safe, unless
-///   there is only a single config and session where it is installed.
+/// - the passed callback function must be safe to call multiple times concurrently
+///   with the same userdata, unless there is only a single config and connection
+///   where it is installed.
 ///
-/// EXPERIMENTAL: this feature of crustls is likely to change in the future, as
+/// EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
 /// the rustls library is re-evaluating their current approach to client hello handling.
 pub type rustls_client_hello_callback = Option<
     unsafe extern "C" fn(
@@ -542,8 +552,8 @@ unsafe impl Sync for ClientHelloResolver {}
 unsafe impl Send for ClientHelloResolver {}
 
 impl rustls_server_config_builder {
-    /// Register a callback to be invoked when a session created from this config
-    /// is seeing a TLS ClientHello message. If `userdata` has been set with
+    /// Register a callback to be invoked when a connection created from this config
+    /// sees a TLS ClientHello message. If `userdata` has been set with
     /// rustls_connection_set_userdata, it will be passed to the callback.
     /// Otherwise the userdata param passed to the callback will be NULL.
     ///
@@ -552,7 +562,7 @@ impl rustls_server_config_builder {
     /// will overwrite the first registration. It is not permitted to pass a NULL
     /// value for `callback`.
     ///
-    /// EXPERIMENTAL: this feature of crustls is likely to change in the future, as
+    /// EXPERIMENTAL: this feature of rustls-ffi is likely to change in the future, as
     /// the rustls library is re-evaluating their current approach to client hello handling.
     /// Installing a client_hello callback will replace any configured certified keys
     /// and vice versa. Same holds true for the set_certified_keys variant.
@@ -624,17 +634,16 @@ pub extern "C" fn rustls_client_hello_select_certified_key(
     ffi_panic_boundary! {
         let hello = try_ref_from_ptr!(hello);
         let schemes: Vec<SignatureScheme> = sigschemes(try_slice!(hello.signature_schemes.data, hello.signature_schemes.len));
-        let out_key: &mut *const rustls_certified_key = unsafe {
-            match out_key.as_mut() {
-                Some(out_key) => out_key,
-                None => return NullParameter,
-            }
-        };
+        if out_key.is_null() {
+            return NullParameter
+        }
         let keys_ptrs: &[*const rustls_certified_key] = try_slice!(certified_keys, certified_keys_len);
         for &key_ptr in keys_ptrs {
             let key_ref: &CertifiedKey = try_ref_from_ptr!(key_ptr);
             if key_ref.key.choose_scheme(&schemes).is_some() {
-                *out_key = key_ptr;
+                unsafe {
+                    *out_key = key_ptr;
+                }
                 return rustls_result::Ok;
             }
         }
@@ -645,7 +654,7 @@ pub extern "C" fn rustls_client_hello_select_certified_key(
 impl rustls_server_config_builder {
     /// Register callbacks for persistence of TLS session IDs and secrets. Both
     /// keys and values are highly sensitive data, containing enough information
-    /// to break the security of the sessions involved.
+    /// to break the security of the connections involved.
     ///
     /// If `userdata` has been set with rustls_connection_set_userdata, it
     /// will be passed to the callbacks. Otherwise the userdata param passed to
